/*******************************************************************************
 * 
 * jKiwi -- A virtual makeover and hairstyler application
 * Copyright (c) 2007-2008 Dan Mihai Ile, Maria Joao Roque Barbado Leal
 * 
 * This file is part of jKiwi.
 * 
 * jKiwi is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * jKiwi is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************/

package tabs;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Widget;

/**
 * Instances of this class represents widget capable
 * of showing two different colors side by side for
 * comparing them before selecting one of the colors
 * <p>
 * Note that although this class is a subclass of <code>Canvas</code>,
 * it does not make sense to add children to it, or set a layout on it.
 * </p>
 */
//TODO a better implementation is needed; check memory leaks
public class ColorDifference extends Composite {

	/**
	 * The current selected color
	 */
	private Color currentColor = null;

	/**
	 * The current selected color in AWT format
	 */
	private java.awt.Color currentAwtColor = null;

	/**
	 * The new color candidate for selection
	 */
	private Color newColor = null;

	/**
	 * Flag to indicate whether the selected color should be disposed on widget dispose event
	 */
	private boolean isCurrentColorDisposable = true;

	/**
	 * Flag to indicate whether the new color should be disposed on widget dispose event
	 */
	private boolean isNewColorDisposable = true;

	/**
	 * Creates a new instance of this class given it's parent composite
	 * and a style value describing its behavior and appearance.
	 * <p>
	 * The style value is either one of the style constants defined in
	 * class <code>SWT</code> which is applicable to instances of this
	 * class, or must be built by <em>bitwise OR</em>'ing together 
	 * (that is, using the <code>int</code> "|" operator) two or more
	 * of those <code>SWT</code> style constants. The class description
	 * lists the style constants that are applicable to the class.
	 * Style bits are also inherited from superclasses.
	 * </p>
	 * 
	 * @param parent the parent for the button
	 * @param style the button's style
	 * 
	 * @see SWT
	 * @see Widget#getStyle
	 */
	public ColorDifference(Composite parent, int style) {
		super(parent, style);
		initialize();
	}

	/**
	 * initialize the color difference
	 */
	private void initialize() {
		setLayout(new GridLayout());
		currentColor = new Color(Display.getCurrent(), 255, 255, 255);
		newColor = new Color(Display.getCurrent(), 255, 255, 255);

		//dispose unnecessary resources when widget disposed
		addDisposeListener(new DisposeListener() {
			public void widgetDisposed(DisposeEvent e) {
				ColorDifference.this.widgetDisposed(e);
			}
		});

		//all the paint for the widget get's done here
		addPaintListener(new PaintListener() {
			public void paintControl(PaintEvent e) {
				GC gc = e.gc;
				int width = getSize().x-1;
				int height = getSize().y-1;

				//draw 2px borders around button
				Color fgColor = new Color(Display.getCurrent(), 177, 165, 152);
				gc.setForeground(fgColor);
				gc.drawRectangle(0, 0, width, height);
				fgColor.dispose();
				fgColor = new Color(Display.getCurrent(), 228, 224, 220);
				gc.setForeground(fgColor);
				gc.drawRectangle(1, 1, width-2, height-2);
				fgColor.dispose();

				//left color (currentColor)
				gc.setBackground(currentColor);
				fgColor = new Color(Display.getCurrent(), getUpperColor(currentColor));
				gc.setForeground(fgColor);
				gc.fillGradientRectangle(2, 2, (width-3)/2, (height-3)/2, true);
				fgColor.dispose();
				fgColor = new Color(Display.getCurrent(), getLowerColor(currentColor));
				gc.setForeground(fgColor);
				gc.fillGradientRectangle(2, 2 + ((height-3)/2), (width-3)/2, (height-2)/2, true);
				fgColor.dispose();

				//right color (newColor)
				gc.setBackground(newColor);
				fgColor = new Color(Display.getCurrent(), getUpperColor(newColor));
				gc.setForeground(fgColor);
				gc.fillGradientRectangle(2 + (width-3)/2, 2, (width-2)/2, (height-3)/2, true);
				fgColor.dispose();
				fgColor = new Color(Display.getCurrent(), getLowerColor(newColor));
				gc.setForeground(fgColor);
				fgColor.dispose();
				gc.fillGradientRectangle(2 + (width-3)/2, 2 + ((height-3)/2), (width-2)/2, (height-2)/2, true);
			}
		});

	}

	/**
	 * Dispose the used resources on widget disposed event
	 */
	private void widgetDisposed(DisposeEvent e) {
		if (isNewColorDisposable) {
			newColor.dispose();
		}
		if (isCurrentColorDisposable) {
			currentColor.dispose();
		}
		currentAwtColor = null;
	}

	/**
	 * Given a color return a brighter RGB color description
	 * @param color the color
	 * @return the brighter RGB color description
	 */
	private RGB getUpperColor(Color color) {
		int newRed = color.getRed() + 10 > 255 ? 255 : color.getRed() + 10;
		int newGreen = color.getGreen() + 10 > 255 ? 255 : color.getGreen() + 10;
		int newBlue = color.getBlue() + 10 > 255 ? 255 : color.getBlue() + 10;
		return new RGB(newRed,newGreen,newBlue);
	}

	/**
	 * Given a color return a darker RGB color description
	 * @param color the color
	 * @return the darker RGB color description
	 */
	private RGB getLowerColor(Color color) {
		int newRed = color.getRed() - 10 < 0 ? 0 : color.getRed() - 10;
		int newGreen = color.getGreen() - 10 < 0 ? 0 : color.getGreen() - 10;
		int newBlue = color.getBlue() - 10 < 0 ? 0 : color.getBlue() - 10;
		return new RGB(newRed,newGreen,newBlue);
	}

	/**
	 * Change this widget's current color
	 * @param currentColor the new color
	 */
	public void setCurrentColor(Color currentColor) {
		if (isCurrentColorDisposable) {
			this.currentColor.dispose();
		}
		isCurrentColorDisposable = false;
		this.currentColor = currentColor;
		this.currentAwtColor = new java.awt.Color(currentColor.getRed(),currentColor.getGreen(),currentColor.getBlue());
		redraw();
	}

	/**
	 * Change the widget's new color candidate
	 * @param newColor the new color candidate
	 */
	public void setNewColor(Color newColor) {
		if (isNewColorDisposable) {
			this.newColor.dispose();
		}
		isNewColorDisposable = false;
		this.newColor = newColor;
		redraw();
	}

	/**
	 * Returns The current widget's color
	 * @return the current widget's color
	 */
	public Color getCurrentColor() {
		return currentColor;
	}

	/**
	 * Returns The current widget's color in AWT format
	 * @return the current widget's color in AWT format
	 */
	public java.awt.Color getAwtCurrentColor() {
		return currentAwtColor;
	}

	/**
	 * Returns The widget's new color candidate
	 * @return the widget's new color candidate
	 */
	public Color getNewColor() {
		return newColor;
	}
}