/*******************************************************************************
 * 
 * jKiwi -- A virtual makeover and hairstyler application
 * Copyright (c) 2007-2008 Dan Mihai Ile, Maria Joao Roque Barbado Leal
 * 
 * This file is part of jKiwi.
 * 
 * jKiwi is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * jKiwi is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************/

package tabs;

import java.awt.Graphics2D;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.image.BufferedImage;

import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;

/**
 * Abstract class that represents a tab on the TabsContainer
 */
public abstract class TabContainer extends Composite implements Comparable<TabContainer> {

	/**
	 * The parent of this tab is TabsContainer
	 */
	protected TabsContainer tabsContainer = null;

	/**
	 * The image that is to be shown on the tab name
	 */
	private Image tabImage = null;

	/**
	 * The text that will appear on the tab name in toolTip
	 */
	private String tabToolTipText = null;

	/**
	 * The position of this tab's layer in the final image
	 */
	private Integer tabLayerPosition = null;

	/**
	 * Default constructor for TabContainer
	 * Note: setTabImage and setTabTooltipText should be called when the new tab is constructed.
	 * @param parent The parent composite
	 * @param style SWT styles
	 * @param tabLayerPosition the tab's layer position on the final image
	 */
	public TabContainer(TabsContainer parent, int style, int tabLayerPosition) {
		//the super of this abstract class sees this class as having the parent an TabFolder and not TabsContainer
		super(parent.getTabFolder(), style);
		this.tabsContainer = parent;
		this.tabLayerPosition = tabLayerPosition;
	}

	/**
	 * Sets the tab image that will be shown on tab name
	 * @param tabImage the tab image
	 */
	protected void setTabImage(Image tabImage) {
		this.tabImage = tabImage;
	}

	/**
	 * Sets the tab name toolTip text
	 * @param tabToolTipText
	 */
	protected void setTabToolTipText(String tabToolTipText) {
		this.tabToolTipText = tabToolTipText;
	}

	/**
	 * Returns the tab name image
	 * @return the tab name image
	 */
	public Image getTabImage() {
		return tabImage;
	}

	/**
	 * Returns the tab name toolTip text
	 * @return toolTip text
	 */
	public String getTabToolTipText() {
		return tabToolTipText;
	}

	/**
	 * Returns the position of this tab's layer on the final image
	 * @return the layer position on the final image
	 */
	public Integer getTabLayerPosition() {
		return tabLayerPosition;
	}

	/**
	 * Implements comparable, so that tabs can be sorted by 
	 * their layer position in the final image
	 */
	public int compareTo(TabContainer otherTab) {

		if (this.tabLayerPosition > otherTab.getTabLayerPosition()) {
			return 1;
		} else if (this.tabLayerPosition < otherTab.getTabLayerPosition()) {
			return -1;
		} else {
			return 0;
		}

	}

	/**
	 * Abstract method to be implemented by every tab
	 * that returns the resulting image which will be drawn
	 * above original image
	 * @return the resulting image layer for this tab
	 */
	public abstract BufferedImage getLayerImage();

	/**
	 * Abstract method to be implemented by every tab
	 * which returns a mouse listener that is notified
	 * when the mouse is in the project's drawing area
	 */
	public abstract MouseListener mouseListener();

	/**
	 * Abstract method to be implemented by every tab
	 * which returns a mouse motion listener that is notified
	 * when the mouse is in the project's drawing area
	 */
	public abstract MouseMotionListener mouseMotionListener();

	/**
	 * Abstract method to be implemented by every tab
	 * that draws the resulting image on the DrawingContainer
	 * using is't g2d
	 * @param g2d DrawingContainer's Graphics2D
	 */
	public abstract void drawCurrentLayer(Graphics2D g2d);

	/**
	 * Abstract method to be implemented by every tab
	 * that returns an Object containing the tab's
	 * current state
	 */
	public abstract Object getCurrentState();

	/**
	 * Abstract method to be implemented by every tab
	 * that changes the current tab state to the new
	 * given tab state
	 */
	public abstract void setCurrentState(Object o);

	/**
	 * Abstract method to be implemented by every tab
	 * that clears the tab's image in the drawing area
	 */
	public abstract void clear();

	/**
	 * Abstract method to be implemented by every tab
	 * that is notified whenever the original drawing
	 * area's image has changed it's position, size
	 * or rotation
	 */
	public abstract void originalImageChanged();
	
	/**
	 * Abstract method to be implemented by every tab
	 * to dispose all unnecessary resources
	 */
	public void dispose() {
		super.dispose();
		disposeData();
	}

	public abstract void disposeData();
}